package net.datacrow.connector;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

import net.datacrow.core.DcConfig;
import net.datacrow.core.server.Connector;
import net.datacrow.core.server.requests.ClientRequest;
import net.datacrow.core.server.response.IServerResponse;

import org.apache.log4j.Logger;

public class ClientRequestHandler {
	
	private static Logger logger = Logger.getLogger(ClientRequestHandler.class);

	private ClientRequest cr;
	
	public ClientRequestHandler(ClientRequest cr) {
		this.cr = cr;
	}
	
	/** 
	 * One method to handle them all.
	 * 
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public IServerResponse process() throws IOException, ClassNotFoundException {
		IServerResponse response = null;
		ObjectInputStream ois = null;
		InputStream is = null;
		ObjectOutputStream oos = null;
		
		try {
			Connector conn = DcConfig.getInstance().getConnector();
			Socket ss = new Socket(conn.getServerAddress(), conn.getApplicationServerPort());
			ss.setKeepAlive(true);
			
			OutputStream os = ss.getOutputStream(); 
			oos = new ObjectOutputStream(os);  
			oos.writeObject(cr);  
			
			is = ss.getInputStream();
			ois = new ObjectInputStream(is);
			response = (IServerResponse) ois.readObject();
			
			logger.debug("Client has received: " + response);
			
			ss.close();  
			
			logger.debug("session ended");

		} finally {
			try {
				if (ois != null) ois.close();
				if (is != null) is.close();
				if (oos != null) oos.close();
			} catch (Exception e) {
				logger.error(e, e);
			}
		}
		
		return response;
	}
}
