/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.datacrow.console.windows.itemforms;

import net.datacrow.console.GUI;
import net.datacrow.core.DcConfig;
import net.datacrow.core.console.IMasterView;
import net.datacrow.core.console.IView;
import net.datacrow.core.modules.DcModules;
import net.datacrow.core.objects.DcObject;
import net.datacrow.core.server.Connector;
import net.datacrow.core.wf.tasks.DeleteItemTask;
import net.datacrow.core.wf.tasks.SaveItemTask;

public class ChildItemForm extends ItemForm {

    private ChildForm childForm;
    
    public ChildItemForm(boolean update, DcObject o, ChildForm childForm) {
        super(false, update, o, true);
        this.childForm = childForm;
    }
    
    @Override
    protected void saveValues() {
        apply();
        
        if (update && !isChanged())  {
        	GUI.getInstance().displayWarningMessage("msgNoChangesToSave");
        } else {
            SaveItemTask task = new SaveItemTask();
            task.addClient(this);
            task.addItem(dco);
        }
    }  
    
    @Override
    public void close(boolean aftersave) { 
        super.close(aftersave);
        childForm = null;
    }
    
    /**
     * Deletes this item from the database
     */
    @Override
    protected void deleteItem() {
        if (GUI.getInstance().displayQuestion("msgDeleteQuestion")) {
            DeleteItemTask task = new DeleteItemTask();
            task.addItem(dco);
            task.addClient(this);
            task.setModule(moduleIdx);
            
            Connector connector = DcConfig.getInstance().getConnector();
            connector.executeTask(task);
        }
    }

    @Override
    public void notifyTaskCompleted(boolean success, String ID) {
    	if (childForm != null) {
            for (DcObject dco : childForm.getItems()) {
                if (dco.getModule().getParent() != null) {
                	IMasterView mv = GUI.getInstance().getSearchView(dco.getModule().getParent().getIndex());
                	IView vw = mv.getCurrent();
                	vw.loadChildren();
                	vw.refreshQuickView();
                } else {
                	IMasterView mv = GUI.getInstance().getSearchView(DcModules._CONTAINER);
                	IView vw = mv.getCurrent();
                	vw.refreshQuickView();
                }
                break;
            }
    	}
        
    	super.notifyTaskCompleted(success, ID);
        this.close(true);
    }
}
