package net.datacrow.util.launcher;

import java.awt.Desktop;
import java.net.URL;

import net.datacrow.console.GUI;
import net.datacrow.core.DcRepository;
import net.datacrow.core.resources.DcResources;
import net.datacrow.core.utilities.ServerUtilities;
import net.datacrow.settings.DcSettings;

import org.apache.log4j.Logger;

public class URLLauncher extends Launcher {

	private static Logger logger = Logger.getLogger(URLLauncher.class.getName());
	
	private URL url;
	
	public URLLauncher(URL url) {
		super();
		this.url = url;
	}

	@Override
	public void launch() {

		boolean launched = false;
		
		String browserPath = DcSettings.getString(DcRepository.Settings.stBrowserPath);
		
		if (!ServerUtilities.isEmpty(browserPath)) {
			try {
				runCmd(new String[] {browserPath, url.toString()});
				launched = true;
			} catch (Exception e) {
			    logger.debug("Failed to launch "  + browserPath + " " + url.toString(), e);
			}
		}
		
		if (!launched) {
			Desktop desktop = getDesktop();
	        if (desktop != null) {
	            try {
	            	desktop.browse(url.toURI());
	            	launched = true;
	            } catch (Exception exp) {
	            	logger.debug("Could not launch URL using the Dekstop class [" + url + "]", exp);
	            	
	            	if (!ServerUtilities.isEmpty(browserPath))
	            	    GUI.getInstance().displayWarningMessage(DcResources.getText("msgCannotLaunchURLNoBrowserPath", url.toString()));
	            	else 
	            	    GUI.getInstance().displayWarningMessage(DcResources.getText("msgCannotLaunchURL", url.toString()));
	            }
	        }
		}
	}
}
